@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,10 +23,12 @@
 
 #include "DiskArbitration.h"
 
+#include "DiskArbitrationPrivate.h"
 #include "DAInternal.h"
 #include "DAServerUser.h"
 
 #include <pthread.h>
+#include <unistd.h>
 
 CFDictionaryRef kDADiskDescriptionMatchMediaUnformatted   = NULL;
 CFDictionaryRef kDADiskDescriptionMatchMediaWhole         = NULL;
@@ -438,29 +440,35 @@ __private_extern__ void _DARegisterCallback( DASessionRef    session,
                                              CFDictionaryRef match,
                                              CFArrayRef      watch )
 {
-    CFDataRef _match = NULL;
-    CFDataRef _watch = NULL;
+    if ( session )
+    {
+        CFDataRef _match = NULL;
+        CFDataRef _watch = NULL;
 
-    if ( match )  _match = _DASerializeDiskDescription( kCFAllocatorDefault, match );
-    if ( watch )  _watch = _DASerialize( kCFAllocatorDefault, watch );
+        if ( match )  _match = _DASerializeDiskDescription( kCFAllocatorDefault, match );
+        if ( watch )  _watch = _DASerialize( kCFAllocatorDefault, watch );
 
-    _DAServerSessionRegisterCallback( _DASessionGetID( session ),
-                                      ( vm_offset_t            ) address,
-                                      ( vm_offset_t            ) context,
-                                      ( int32_t                ) kind,
-                                      ( int32_t                ) order,
-                                      ( vm_address_t           ) ( _match ? CFDataGetBytePtr( _match ) : 0 ),
-                                      ( mach_msg_type_number_t ) ( _match ? CFDataGetLength(  _match ) : 0 ),
-                                      ( vm_address_t           ) ( _watch ? CFDataGetBytePtr( _watch ) : 0 ),
-                                      ( mach_msg_type_number_t ) ( _watch ? CFDataGetLength(  _watch ) : 0 ) );
+        _DAServerSessionRegisterCallback( _DASessionGetID( session ),
+                                          ( vm_offset_t            ) address,
+                                          ( vm_offset_t            ) context,
+                                          ( int32_t                ) kind,
+                                          ( int32_t                ) order,
+                                          ( vm_address_t           ) ( _match ? CFDataGetBytePtr( _match ) : 0 ),
+                                          ( mach_msg_type_number_t ) ( _match ? CFDataGetLength(  _match ) : 0 ),
+                                          ( vm_address_t           ) ( _watch ? CFDataGetBytePtr( _watch ) : 0 ),
+                                          ( mach_msg_type_number_t ) ( _watch ? CFDataGetLength(  _watch ) : 0 ) );
 
-    if ( _match )  CFRelease( _match );
-    if ( _watch )  CFRelease( _watch );
+        if ( _match )  CFRelease( _match );
+        if ( _watch )  CFRelease( _watch );
+    }
 }
 
 __private_extern__ void _DAUnregisterCallback( DASessionRef session, void * address, void * context )
 {
-    _DAServerSessionUnregisterCallback( _DASessionGetID( session ), ( vm_offset_t ) address, ( vm_offset_t ) context );
+    if ( session )
+    {
+        _DAServerSessionUnregisterCallback( _DASessionGetID( session ), ( vm_offset_t ) address, ( vm_offset_t ) context );
+    }
 }
 
 void DADiskClaim( DADiskRef                  disk,
@@ -474,23 +482,31 @@ void DADiskClaim( DADiskRef                  disk,
     CFNumberRef _releaseContext;
     DAReturn    status;
 
-    _release        = ___CFNumberCreateWithIntegerValue( kCFAllocatorDefault, ( vm_offset_t ) release        );
-    _releaseContext = ___CFNumberCreateWithIntegerValue( kCFAllocatorDefault, ( vm_offset_t ) releaseContext );
+    status = kDAReturnBadArgument;
 
-    status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskClaim, disk, options, _release, _releaseContext, callback, callbackContext );
+    if ( disk )
+    {
+        _release        = ___CFNumberCreateWithIntegerValue( kCFAllocatorDefault, ( vm_offset_t ) release        );
+        _releaseContext = ___CFNumberCreateWithIntegerValue( kCFAllocatorDefault, ( vm_offset_t ) releaseContext );
 
-    if ( _release        )  CFRelease( _release        );
-    if ( _releaseContext )  CFRelease( _releaseContext );
+        status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskClaim, disk, options, _release, _releaseContext, callback, callbackContext );
+
+        if ( _release        )  CFRelease( _release        );
+        if ( _releaseContext )  CFRelease( _releaseContext );
+    }
 
     if ( status )
     {
-        DADissenterRef dissenter;
+        if ( callback )
+        {
+            DADissenterRef dissenter;
 
-        dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
+            dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
 
-        ( callback )( disk, dissenter, callbackContext );
+            ( callback )( disk, dissenter, callbackContext );
 
-        CFRelease( dissenter );
+            CFRelease( dissenter );
+        }
     }
 }
 
@@ -498,39 +514,57 @@ void DADiskEject( DADiskRef disk, DADiskEjectOptions options, DADiskEjectCallbac
 {
     DAReturn status;
 
-    status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightUnmount );
+    status = kDAReturnBadArgument;
 
-    if ( status == kDAReturnSuccess )
+    if ( disk )
     {
-        status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskEject, disk, options, NULL, NULL, callback, context );
+        status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightUnmount );
+
+        if ( status == kDAReturnSuccess )
+        {
+            status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskEject, disk, options, NULL, NULL, callback, context );
+        }
     }
 
     if ( status )
     {
-        DADissenterRef dissenter;
+        if ( callback )
+        {
+            DADissenterRef dissenter;
 
-        dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
+            dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
 
-        ( callback )( disk, dissenter, context );
+            ( callback )( disk, dissenter, context );
 
-        CFRelease( dissenter );
+            CFRelease( dissenter );
+        }
     }
 }
 
 DADiskOptions DADiskGetOptions( DADiskRef disk )
 {
-    int32_t options = 0;
+    int32_t options;
 
-    _DAServerDiskGetOptions( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), &options );
+    options = kDADiskOptionDefault;
+
+    if ( disk )
+    {
+        _DAServerDiskGetOptions( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), &options );
+    }
 
     return options;
 }
 
 Boolean DADiskIsClaimed( DADiskRef disk )
 {
-    boolean_t claimed = FALSE;
+    boolean_t claimed;
 
-    _DAServerDiskIsClaimed( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), &claimed );
+    claimed = FALSE;
+
+    if ( disk )
+    {
+        _DAServerDiskIsClaimed( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), &claimed );
+    }
 
     return claimed;
 }
@@ -590,11 +624,16 @@ void DADiskMountWithArguments( DADiskRef           disk,
         }
     }
 
-    status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightMount );
+    status = kDAReturnBadArgument;
 
-    if ( status == kDAReturnSuccess )
+    if ( disk )
     {
-        status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskMount, disk, options, path ? CFURLGetString( path ) : NULL, argument, callback, context );
+        status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightMount );
+
+        if ( status == kDAReturnSuccess )
+        {
+            status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskMount, disk, options, path ? CFURLGetString( path ) : NULL, argument, callback, context );
+        }
     }
 
     if ( argument )
@@ -609,13 +648,16 @@ void DADiskMountWithArguments( DADiskRef           disk,
 
     if ( status )
     {
-        DADissenterRef dissenter;
+        if ( callback )
+        {
+            DADissenterRef dissenter;
 
-        dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
+            dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
 
-        ( callback )( disk, dissenter, context );
+            ( callback )( disk, dissenter, context );
 
-        CFRelease( dissenter );
+            CFRelease( dissenter );
+        }
     }
 }
 
@@ -625,56 +667,79 @@ void DADiskRename( DADiskRef disk, CFStringRef name, DADiskRenameOptions options
 
     status = kDAReturnBadArgument;
 
-    if ( name )
+    if ( disk )
     {
-        if ( CFGetTypeID( name ) == CFStringGetTypeID( ) )
+        if ( name )
         {
-            status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightRename );
-
-            if ( status == kDAReturnSuccess )
+            if ( CFGetTypeID( name ) == CFStringGetTypeID( ) )
             {
-                status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskRename, disk, options, name, NULL, callback, context );
+                status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightRename );
+
+                if ( status == kDAReturnSuccess )
+                {
+                    status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskRename, disk, options, name, NULL, callback, context );
+                }
             }
         }
     }
 
     if ( status )
     {
-        DADissenterRef dissenter;
+        if ( callback )
+        {
+            DADissenterRef dissenter;
 
-        dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
+            dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
 
-        ( callback )( disk, dissenter, context );
+            ( callback )( disk, dissenter, context );
 
-        CFRelease( dissenter );
+            CFRelease( dissenter );
+        }
     }
 }
 
 DAReturn DADiskSetOptions( DADiskRef disk, DADiskOptions options, Boolean value )
 {
-    return _DAServerDiskSetOptions( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), options, value );
+    DAReturn status;
+
+    status = kDAReturnBadArgument;
+
+    if ( disk )
+    {
+        status = _DAServerDiskSetOptions( _DADiskGetSessionID( disk ), _DADiskGetID( disk ), options, value );
+    }
+
+    return status;
 }
 
 void DADiskUnmount( DADiskRef disk, DADiskUnmountOptions options, DADiskUnmountCallback callback, void * context )
 {
     DAReturn status;
 
-    status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightUnmount );
+    status = kDAReturnBadArgument;
 
-    if ( status == kDAReturnSuccess )
+    if ( disk )
     {
-        status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskUnmount, disk, options, NULL, NULL, callback, context );
+        status = _DAAuthorize( _DADiskGetSession( disk ), disk, _kDAAuthorizeRightUnmount );
+
+        if ( status == kDAReturnSuccess )
+        {
+            status = __DAQueueRequest( _DADiskGetSession( disk ), _kDADiskUnmount, disk, options, NULL, NULL, callback, context );
+        }
     }
 
     if ( status )
     {
-        DADissenterRef dissenter;
+        if ( callback )
+        {
+            DADissenterRef dissenter;
 
-        dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
+            dissenter = DADissenterCreate( kCFAllocatorDefault, status, NULL );
 
-        ( callback )( disk, dissenter, context );
+            ( callback )( disk, dissenter, context );
 
-        CFRelease( dissenter );
+            CFRelease( dissenter );
+        }
     }
 }
 
@@ -738,7 +803,10 @@ void DARegisterDiskUnmountApprovalCallback( DAApprovalSessionRef          sessio
 
 void DADiskUnclaim( DADiskRef disk )
 {
-    _DAServerDiskUnclaim( _DADiskGetSessionID( disk ), _DADiskGetID( disk ) );
+    if ( disk )
+    {
+        _DAServerDiskUnclaim( _DADiskGetSessionID( disk ), _DADiskGetID( disk ) );
+    }
 }
 
 void DAUnregisterCallback( DASessionRef session, void * callback, void * context )
