@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -291,8 +291,6 @@ static void __DAStageDispatch( void * info )
 
         DAIdleCallback( );
 
-        DAMainRendezvous( );
-
         if ( gDAConsoleUser )
         {
 ///w:start
@@ -315,40 +313,37 @@ static void __DAStageDispatch( void * info )
 
                 if ( DADiskGetDescription( disk, kDADiskDescriptionMediaWholeKey ) == kCFBooleanTrue )
                 {
-                    if ( DADiskGetOption( disk, kDADiskOptionMountAutomatic ) )
+                    if ( DAUnitGetState( disk, kDAUnitStateStagedUnreadable ) == FALSE )
                     {
-                        if ( DAUnitGetState( disk, kDAUnitStateStagedUnreadable ) == FALSE )
-                        {
 ///w:start
-                            io_service_t media;
+                        io_service_t media;
 
-                            media = DADiskGetIOMedia( disk );
+                        media = DADiskGetIOMedia( disk );
 
-                            if ( media )
+                        if ( media )
+                        {
+                            CFTypeRef object;
+
+                            object = IORegistryEntrySearchCFProperty( media,
+                                                                      kIOServicePlane,
+                                                                      CFSTR( "image-path" ),
+                                                                      kCFAllocatorDefault,
+                                                                      kIORegistryIterateParents | kIORegistryIterateRecursively );
+
+                            if ( object )
                             {
-                                CFTypeRef object;
+                                CFRelease( object );
 
-                                object = IORegistryEntrySearchCFProperty( media,
-                                                                          kIOServicePlane,
-                                                                          CFSTR( "image-path" ),
-                                                                          kCFAllocatorDefault,
-                                                                          kIORegistryIterateParents | kIORegistryIterateRecursively );
-
-                                if ( object )
-                                {
-                                    CFRelease( object );
-
-                                    continue;
-                                }
+                                continue;
                             }
-///w:stop
-                            if ( _DAUnitIsUnreadable( disk ) )
-                            {
-                                DADialogShowDeviceUnreadable( disk );
-                            }
-
-                            DAUnitSetState( disk, kDAUnitStateStagedUnreadable, TRUE );
                         }
+///w:stop
+                        if ( _DAUnitIsUnreadable( disk ) )
+                        {
+                            DADialogShowDeviceUnreadable( disk );
+                        }
+
+                        DAUnitSetState( disk, kDAUnitStateStagedUnreadable, TRUE );
                     }
                 }
             }
@@ -378,8 +373,6 @@ static void __DAStageMount( DADiskRef disk )
 
         DAUnitSetState( disk, kDAUnitStateCommandActive, TRUE );
 
-	DAUnitSetState( disk, kDAUnitStateEjected, FALSE );
-
         DALogDebug( "  mounted disk, id = %@, ongoing.", disk );
 
         DAMountWithArguments( disk, NULL, __DAStageMountCallback, disk, CFSTR( "automatic" ), NULL );
@@ -800,6 +793,11 @@ static void __DAStageProbeCallback( int status, CFBooleanRef clean, CFStringRef
 
                             DADiskSetFileSystem( disk, filesystem );
 
+                            if ( CFDictionaryGetValue( candidate, CFSTR( "autodiskmount" ) ) == kCFBooleanFalse )
+                            {
+                                DADiskSetOptions( disk, kDADiskOptionMountAutomatic | kDADiskOptionMountAutomaticNoDefer, FALSE );
+                            }
+
                             CFArrayRemoveValueAtIndex( candidates, 0 );
 
                             DALogDebug( "  probed disk, id = %@, with %@, ongoing.", disk, kind );
@@ -983,6 +981,9 @@ static void __DAStageRepair( DADiskRef disk )
 
                         CFRelease( path );
                     }
+
+                    DADiskSetOption( disk, kDADiskOptionMountAutomatic,        TRUE );
+                    DADiskSetOption( disk, kDADiskOptionMountAutomaticNoDefer, TRUE );
                 }
 
                 DADiskSetState( disk, kDADiskStateRequireRepair,       FALSE );
